<?php

/**
 * This function should include all classes and functions that access the database.
 * In most BuddyPress components the database access classes are treated like a model,
 * where each table has a class that can be used to create an object populated with a row
 * from the corresponding database table.
 * 
 * By doing this you can easily save, update and delete records using the class, you're also
 * abstracting database access.
 */

class BP_CubePoint_Point {
	var $id;
	var $uid;
    var $timestamp;
    var $type;
	var $points;
    var $source;
	
	/**
	 * bp_cubepoint_point()
	 *
	 * This is the constructor, it is auto run when the class is instantiated.
	 * It will either create a new empty object if no ID is set, or fill the object
	 * with a row from the table if an ID is provided.
	 */
	function BP_CubePoint_Point( $id = null ) {
		$this->__construct( $id );
	}
	
	function __construct( $id = null ) {
		global $wpdb, $bp;	
		
		if ( $id ) {	
			$this->populate( $id );
		}
	}
	
	/**
	 * populate()
	 *
	 * This method will populate the object with a row from the database, based on the
	 * ID passed to the constructor.
	 */
	function populate($id) {
		global $wpdb,$bp;
		
		$sql = $wpdb->prepare( "SELECT * FROM {$bp->cubepoint->table_name} WHERE id = %d", $id );
		$point = $wpdb->get_row( $sql );
		
		if ( $point ) {
			$this->uid = $point->uid;
			$this->id = $point->id;
	        $this->timestamp = $point->timestamp;
	        $this->type = $point->type;
	        $this->source = $point->source;
	        $this->points = $point->points;
		}
	}
	
	/**
	 * save()
	 *
	 * This method will save an object to the database. It will dynamically switch between
	 * INSERT and UPDATE depending on whether or not the object already exists in the database.
	 */
	
	function save() {
		global $wpdb, $bp;
		
		/***
		 * In this save() method, you should add pre-save filters to all the values you are saving to the
		 * database. This helps with two things -
		 * 
		 * 1. Blanket filtering of values by plugins (for example if a plugin wanted to force a specific 
		 *	  value for all saves)
		 * 
		 * 2. Security - attaching a wp_filter_kses() call to all filters, so you are not saving
		 *	  potentially dangerous values to the database.
		 *
		 * It's very important that for number 2 above, you add a call like this for each filter to
		 * 'bp-cubepoint-filters.php'
		 *
		 *   add_filter( 'example_data_fieldname1_before_save', 'wp_filter_kses' );
		 */			
		/* Call a before save action here */
		do_action( 'bp_cubepoint_data_before_save', $this );

		// Don't try and save if there is no user ID.
		if ( !$this->uid)
			return false;

        if ( $this->id ) {
			// Update
			$sql = $wpdb->prepare(
				"UPDATE {$bp->cubepoint->table_name} SET
					uid = %d,
					timestamp = %s,
					type = %s,
					points = %s,
					source =%s
				WHERE id = %d",
					$this->uid,
					$this->timestamp,
					$this->type,
					$this->points,
					$this->source,
					$this->id
				);
		} else {
			// Save
			$sql = $wpdb->prepare(
					"INSERT INTO {$bp->cubepoint->table_name} (
						uid,
						timestamp,
						type,
						points,
						source
					) VALUES (
						%s, %d, %s, %s, %s
					)",
						$this->uid,
						$this->timestamp,
						$this->type,
						$this->points,
						$this->source
					);
		}
		
		$result = $wpdb->query( $sql );
	    
		if ( !$result )
			return false;
		
		if ( !$this->id ) {
			$this->id = $wpdb->insert_id;
		}	
		
		/* Add an after save action here */
		do_action( 'bp_cubepoint_data_after_save', $this ); 
		
		return $result;
	}

	/**
	 * delete()
	 *
	 * This method will delete the corresponding row for an object from the database.
	 */	
	function delete() {
		global $wpdb, $bp;
		
		return $wpdb->query( $wpdb->prepare( "DELETE FROM {$bp->cubepoint->table_name} WHERE id = %d", $this->id ) );
	}

	/* Static Functions */

	/**
	 * Static functions can be used to bulk delete items in a table, or do something that
	 * doesn't necessarily warrant the instantiation of the class.
	 *
	 * Look at bp-core-classes.php for examples of mass delete.
	 */

	/*function delete_all() {

	}*/
	
	public static function query_points($args = '',$count=false) {
		global $bp, $wpdb;
		
		$defaults = bp_cubepoint_default_query_args();
		
		$r = apply_filters('bp_cubepoint_query_args',wp_parse_args( $args, $defaults ));
		extract( $r , EXTR_SKIP);
		
		$where = "1 = 1";
		
		if ($uid){
		$where .= $wpdb->prepare(' AND uid = %d',$uid);	
		}
		
		if(!$count){	
		$order = "";	
		$limits = "";
		if(!$id){
			if ($orderkey != 'id' && $orderkey != 'uid' && $orderkey != 'type') $orderkey = 'id';
				if ($ordersort != 'DESC' && $ordersort != 'ASC') $ordersort = 'ASC';
				$order = "ORDER BY $orderkey $ordersort";
				if ($per_page){
					if ( empty($offset) ) {
						$limits = $wpdb->prepare('LIMIT %d, %d', ($page-1)*$per_page , $per_page);
					} else { // we're ignoring $page and using 'offset'
						$limits = $wpdb->prepare('LIMIT %d, %d', $offset , $per_page);
					}
				}
			}
			$sql = $wpdb->prepare( "SELECT * FROM {$bp->cubepoint->table_name} WHERE $where $order $limits") ;
			$result = $wpdb->get_results( $sql );
			
		} else {
		
			$sql =  $wpdb->prepare( "SELECT DISTINCT $select COUNT(id) AS count FROM {$bp->cubepoint->table_name} WHERE $where") ;
			
			$result = $wpdb->get_var( $sql );
		}

		return $result;	
	}
	
}

function bp_cubepoint_default_query_args(){
	global $bp;
	$args = array();
	
	$args['uid'] = $bp->displayed_user->id ? $bp->displayed_user->id : false;
	$args['id'] = false;
	$args['page']=1;
	$args['per_page']=$bp->cubepoint->bp_cubepoint_per_page;
	$args['max']=false;
	$args['ordersort']='DESC';
	$args['orderkey']='id';
	
	if($bp->cubepoint->points_slug == $bp->current_action){
		$args['page'] = ( isset($bp->action_variables[0]) && (string)(int) $bp->action_variables[0] === (string) $bp->action_variables[0] ) ? (int) $bp->action_variables[0] : 1 ;
	}
	return $args;
}

?>